from . import utils

from . import types

from . import props

from . import path

from . import ops

from . import msgbus

from . import context

from . import app

import typing

data: types.BlendData = ...

"""

Access to Blender's internal data

"""
