"""


Additional Math Functions (bl_math)
***********************************

Miscellaneous math utilities module

:func:`clamp`

"""

import typing

def clamp(value: float, min: float = 0, max: float = 1) -> float:

  """

  Clamps the float value between minimum and maximum. To avoid
confusion, any call must use either one or all three arguments.

  """

  ...
