"""


Gizmogroup Operators
********************

:func:`gizmo_select`

:func:`gizmo_tweak`

"""

import typing

def gizmo_select(extend: bool = False, deselect: bool = False, toggle: bool = False, deselect_all: bool = False) -> None:

  """

  Select the currently highlighted gizmo

  """

  ...

def gizmo_tweak() -> None:

  """

  Tweak the active gizmo

  """

  ...
