"""


Simulation Operators
********************

:func:`new`

"""

import typing

def new() -> None:

  """

  Create a new simulation data block and edit it in the opened simulation editor

  """

  ...
