"""


Geometry Operators
******************

:func:`attribute_add`

:func:`attribute_remove`

"""

import typing

def attribute_add(name: str = 'Attribute', data_type: str = 'FLOAT', domain: str = 'POINT') -> None:

  """

  Add attribute to geometry

  """

  ...

def attribute_remove() -> None:

  """

  Remove attribute from geometry

  """

  ...
