"""


Import Anim Operators
*********************

:func:`bvh`

"""

import typing

def bvh(filepath: str = '', filter_glob: str = '*.bvh', target: str = 'ARMATURE', global_scale: float = 1.0, frame_start: int = 1, use_fps_scale: bool = False, update_scene_fps: bool = False, update_scene_duration: bool = False, use_cyclic: bool = False, rotate_mode: str = 'NATIVE', axis_forward: str = '-Z', axis_up: str = 'Y') -> None:

  """

  Load a BVH motion capture file

  """

  ...
