"""


GPU Select (gpu.select)
***********************

This module provides access to selection.

:func:`load_id`

"""

import typing

def load_id(id: typing.Any) -> None:

  """

  Set the selection ID.

  """

  ...
